package nl.amis.adffaces.tree;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import java.util.Map;

import javax.faces.application.Application;
import javax.faces.context.FacesContext;

import javax.faces.el.ValueBinding;

import oracle.adf.model.binding.DCIteratorBinding;
import oracle.adf.view.faces.model.TreeModel;

import oracle.binding.BindingContainer;

import oracle.jbo.Row;
import oracle.jbo.RowSetIterator;
import oracle.jbo.domain.Number;

public class HierarchicalQueryTreeModel {

    private String hierarchicalIteratorName;
    private String rootLabel;
    private TreeModel treemodel;

    public HierarchicalQueryTreeModel() {
    }


    private TreeNode findParent(List<TreeNode> nodes, int level) {
        // get last of the rootnodes
        TreeNode tn = nodes.get(nodes.size() - 1);
        // the parent for this node will be one level higher than the parameter level;
        // so if level = 2, than the parent will be on level 1 (rootlevel); if the level is 3, 
        // the parent will be on level 2; the parent is the last node at the parentlevel
        for (int i = level; i > 2; i--) {
            // get lastchild of current node
            tn = (TreeNode)tn.getChildren().toArray()[tn.getChildCount() - 1];
        }
        return tn;
    }

    private TreeModel initializeTreeModel() {
        // create the list of root nodes:
        List nodes = new ArrayList();
        //    TreeNode root = new TreeNode(getRootLabel(), "root");


        DCIteratorBinding hierarchicalIterator = 
            (DCIteratorBinding)getBindings().get(hierarchicalIteratorName);
        RowSetIterator hierarchicalRSI = 
            hierarchicalIterator.getRowSetIterator();
        Row currentNode = hierarchicalRSI.first();
        boolean firstNode = true;
        do {
            // if the level of a node is larger than the previous one, it is a child

            if (!firstNode)
                currentNode = hierarchicalRSI.next();
            else
                firstNode = false;

            TreeNode treenode = 
                new TreeNode((String)currentNode.getAttribute("NodeLabel"), 
                             (String)currentNode.getAttribute("NodeType"));
            treenode.setLevel(((Number)currentNode.getAttribute("Level")).intValue());
            Map attributes = new HashMap();
            // copy all attributes from the Row to the Node
            // note: the Row will only be available as long as we are on a page
            // that has the iterator instantiated (which is basically only right at the
            // time of instantiating the TreeModel
            for (int i = 0; i < currentNode.getAttributeCount(); i++) {
                attributes.put(currentNode.getAttributeNames()[i], 
                               currentNode.getAttribute(i));
            }
            treenode.setAttributes(attributes);
            if (treenode.getLevel() == 1) {
                nodes.add(treenode);
            } else {
                findParent(nodes, treenode.getLevel())
                                 .getChildren().add(treenode);
            }
        } while (hierarchicalRSI.hasNext()); // Master
        return new SpecialTreeModel(nodes, "children");

    }

    public BindingContainer getBindings() {
        FacesContext fc = FacesContext.getCurrentInstance();
        Application app = fc.getApplication();
        ValueBinding vb = app.createValueBinding("#{bindings}");
        return (BindingContainer)vb.getValue(fc);
    }

    public void setHierarchicalIteratorName(String hierarchicalIteratorName) {
        this.hierarchicalIteratorName = hierarchicalIteratorName;
    }

    public String getHierarchicalIteratorName() {
        return hierarchicalIteratorName;
    }

    public void setRootLabel(String rootLabel) {
        this.rootLabel = rootLabel;
    }

    public String getRootLabel() {
        return rootLabel;
    }


    public TreeModel getTreemodel() {
        if (treemodel == null) {
            treemodel = initializeTreeModel();

        }
        return treemodel;
    }
}
